from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, './README.md'), encoding='utf-8') as f:
    long_description = f.read()

"""
Build Info:
python3 -m build
twine upload dist/*
"""


setup(
    name='blankly-slate',
    packages=find_packages(),
    version='v1.10.2-beta',
    license='mit',
    description='View, manage and share your model from any codebase with slate',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='blankly',
    url='https://blankly.finance',
    entry_points={'console_scripts': ['slate = slate.cli:main']},
    install_requires=[
        'questionary >= 1.10.0',
        'yaspin >= 2.1.0',
        'requests',
        'pandas',
        'numpy',
    ],
    classifiers=[
        # Possible: "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10'
    ],
)
